/*
 * Copyright 1993-2006 NVIDIA Corporation.  All rights reserved.
 *
 * NOTICE TO USER:
 *
 * This source code is subject to NVIDIA ownership rights under U.S. and
 * international Copyright laws.
 *
 * This software and the information contained herein is PROPRIETARY and
 * CONFIDENTIAL to NVIDIA and is being provided under the terms and
 * conditions of a Non-Disclosure Agreement.  Any reproduction or
 * disclosure to any third party without the express written consent of
 * NVIDIA is prohibited.
 *
 * NVIDIA MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS SOURCE
 * CODE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR
 * IMPLIED WARRANTY OF ANY KIND.  NVIDIA DISCLAIMS ALL WARRANTIES WITH
 * REGARD TO THIS SOURCE CODE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
 * IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
 * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOURCE CODE.
 *
 * U.S. Government End Users.  This source code is a "commercial item" as
 * that term is defined at 48 C.F.R. 2.101 (OCT 1995), consisting  of
 * "commercial computer software" and "commercial computer software
 * documentation" as such terms are used in 48 C.F.R. 12.212 (SEPT 1995)
 * and is provided to the U.S. Government only as a commercial end item.
 * Consistent with 48 C.F.R.12.212 and 48 C.F.R. 227.7202-1 through
 * 227.7202-4 (JUNE 1995), all U.S. Government End Users acquire the
 * source code with only those rights set forth herein.
 */

/* Matrix multiplication: P = M * N.
 * Device code.
 */

#ifndef _MATRIXMUL_KERNEL_H_
#define _MATRIXMUL_KERNEL_H_

#include "matrixmul.h"

////////////////////////////////////////////////////////////////////////////////
//! Simple test kernel for device functionality
//! @param g_idata  input data in global memory
//! @param g_odata  output data in global memory
////////////////////////////////////////////////////////////////////////////////

    __global__ void
matrixMul(
    float* P, const float* M, const float* N,
    const int Mh, const int Mw, const int Nw,
    const int block_size)
{
    const int bx = blockIdx.x;
    const int by = blockIdx.y;

    const int tx = threadIdx.x;
    const int ty = threadIdx.y;

    float Psub = 0;
    int i = 0, indexM = 0, indexN = 0, indexP = 0;

    // ===================================================================
    // Begin solution part 5
    // Determine the output index of each thread.
    // Compute the dot product of one row of M and one column of N
    // for each thread.
    // Write the computed value to matrix P at the correct index.
    // ===================================================================

    // Index of the first element of M loaded by this thread by the block
    indexM = Mw * (by * block_size + ty);

    // Index of the first element of N processed by the block
    indexN = bx * block_size + tx;

    // Destination matrix index
    // Set indexP to reference the output element of this thread
    indexP = indexM + indexN;

    #ifdef emu
    //printf ("b (y,x): (%d, %d), t (y,x): (%d, %d), ", by,bx,ty,tx);
    //printf ("index (M,N): (%2d, %2d) ", indexM, indexN);
    //printf ("indexP: %2d\n", indexP);
    #endif

    // For each index from [0, Width of M]
    for (i = 0; i < Mw; i++) {

        // Multiply the corresponding elements of M and N, and accumulate
        // into partial sum Psub.
        Psub += M[indexM] * N[indexN];
        //Psub += M[Mw * (by * block_size + ty)] * N[bx * block_size + tx];
        #ifdef emu
        //printf ("                                  index (M,N): ");
        //printf ("(%2d, %2d)\n", indexM, indexN);
        #endif

        // Update indexes into M and N for next iteration
        indexM++;
        indexN += Nw;
    }
    P[indexP] = Psub;

    #ifdef emu
    //printf ("P[%d] = Psub: %f = %f\n", indexP, P[indexP], Psub);
    #endif

    // End of solution part 5 ============================================
}

#endif // #ifndef _MATRIXMUL_KERNEL_H_


