#ifndef _REDUCTION_KERNEL_CU_
#define _REDUCTION_KERNEL_CU_

// includes, kernels
#include <assert.h>
#include <cutil_inline.h>

// You can use any other block size you wish.
#define BLOCK_SIZE 256 

// **===-------- Lab 2.2 - Modify the body of this function -----------===**
// Reduction - Kernel 
__global__ void reduction_step(float *output, float *input, int nr_elm)
{
    __shared__  float scratch[BLOCK_SIZE*2];

    int tid = threadIdx.x;
    int stride;
    // Coalesced load to SHM
    int gidx = (blockIdx.x * BLOCK_SIZE * 2 + threadIdx.x);
    scratch[tid]   = gidx < nr_elm ? input[gidx]:0.0f;
    scratch[tid+BLOCK_SIZE] = (gidx+BLOCK_SIZE) < nr_elm ? input[gidx+BLOCK_SIZE]:0.0f;
    __syncthreads();
    // build the sum tree (in-place)
    for (stride = BLOCK_SIZE; stride > 0; stride >>= 1) {
	if (threadIdx.x < stride) {
	    scratch[threadIdx.x] += scratch[threadIdx.x + stride];
	}
	__syncthreads();
    }
    // write results to global memory
    if(threadIdx.x == 0)
	output[blockIdx.x] = scratch[0];
}

// **===-------- LAB 2.2 - Modify the body of this function -----------===**
// You may need to make multiple kernel calls, make your own kernel
// function in this file, and then call them from here.
void reductionArray(float *outArray, float *inArray, int numElements)
{
    dim3 grid((numElements + BLOCK_SIZE * 2 -1)/(BLOCK_SIZE *2), 1, 1);
    dim3 block(BLOCK_SIZE, 1, 1);
    int i=0;

    for (i = numElements; i > 1; i = grid.x) {

	grid.x = (i + BLOCK_SIZE * 2 - 1)/(BLOCK_SIZE * 2);
	reduction_step<<<grid, block>>>(outArray,
		i == numElements ?inArray:outArray, i); 
	cutilCheckMsg("reduction_step() execution failed\n");
    }
}
// **===-----------------------------------------------------------===**


#endif // _REDUCTION_KERNEL_CU_
